@@ -1,18 +1,18 @@
 /**
- *   (c) 2001-2006 Nathan Hjelm <hjelmn@users.sourceforge.net>
- *   v1.5.0 rioi.h
+ *   (c) 2001-2012 Nathan Hjelm <hjelmn@users.sourceforge.net>
+ *   v1.5.3 rioi.h
  *
  *   header file for librioutil internal functions
  *   
  *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   it under the terms of the GNU Lesser General Public License as published
+ *   by the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
  *   
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ *   GNU Lesser General Public License for more details.
  *   
  *   You should have received a copy of the GNU Library Public License
  *   along with this program; if not, write to the Free Software
@@ -182,13 +182,13 @@ Group ID                0x7c   ????            ????               64
 #include <endian.h>
 #include <byteswap.h>
 
-#elif defined (__MacOSX__)
+#elif defined (__APPLE__)
 
 #include <architecture/byte_order.h>
 
-#define bswap_64(x) NXSwapLongLong(x)
-#define bswap_32(x) NXSwapLong(x)
-#define bswap_16(x) NXSwapShort(x)
+#define bswap_64(x) OSSwapInt64(x)
+#define bswap_32(x) OSSwapInt32(x)
+#define bswap_16(x) OSSwapInt16(x)
 
 #elif defined (__NetBSD__)
 
@@ -576,15 +576,30 @@ typedef struct _riot_rio_prefs {
   u_int8_t	unk13[1952];
 } riot_prefs_t;
 
+/*                                                                                                                                                     Playlist format used on older players (.lst file format):
+
+  Notes:
+   - The first entry is always blank.
+   - The name is the same as the rio_file->name of the song.
+   - The title is the first 32 characters of rio_file->title.
+*/
+typedef struct _rio_lst {
+  struct song_list {
+    u_int8_t  name[64];
+    u_int32_t unk0[3];
+    u_int32_t entry;
+    u_int32_t unk1[4];
+    u_int8_t  title[32];
+  } playlist[128];
+} rio_plst_t;
+
+
 /***
 
   Internal Functions
 
 ***/
 
-void rio_log      (rios_t *rio, int error, char *format, ...);
-void rio_log_data (rios_t *rio, char *dir, unsigned char *data, int data_size);
-
 int  wake_rio     (rios_t *rio);
 int  try_lock_rio (rios_t *rio);
 void unlock_rio   (rios_t *rio);
@@ -592,8 +607,8 @@ void unlock_rio   (rios_t *rio);
 /* rio.c : used to build a rios_t */
 int get_file_info_rio (rios_t *rio, rio_file_t *file, u_int8_t memory_unit, u_int16_t file_no);
 int get_memory_info_rio (rios_t *rio, rio_mem_t *memory, u_int8_t memory_unit);
+int generate_mem_list_rio (rios_t *rio);
 
-void free_info_rio (rios_t *rio);
 int return_generation_rio (rios_t *rio);
 int return_type_rio(rios_t *rio);
 void update_free_intrn_rio (rios_t *rio, u_int8_t memory_unit);
@@ -608,20 +623,22 @@ int send_command_rio (rios_t *rio, int request, int value, int index);
 
 /* id3.c */
 int get_id3_info (char *file_name, rio_file_t *mp3_file);
+int id3v2_size (unsigned char data[14]);
 
-/* mp3.c, downloadable.c, playlist.c */
+/* mp3.c, downloadable.c */
 int mp3_info (info_page_t *newInfo, char *file_name);
 int downloadable_info (info_page_t *newInfo, char *file_name);
-int playlist_info (info_page_t *newInfo, char *file_name);
-int new_playlist_info (info_page_t *newInfo, char *file_name, char *name);
 
 /* file_list.c */
 int generate_flist_riomc (rios_t *rio, u_int8_t memory_unit);
 int generate_flist_riohd (rios_t *rio);
 int flist_add_rio (rios_t *rio, int memory_unit, info_page_t info);
-int flist_remove_rio (rios_t *rio, int memory_unit, int file_no);
+int flist_remove_rio (rios_t *rio, uint memory_unit, uint file_no);
+int flist_get_file_name_rio (rios_t *rio, uint memory_unit, uint file_no, char *file_namep, int file_name_len);
+int flist_get_file_id_rio (rios_t *rio, uint memory_unit, uint file_no);
 int size_flist_rio (rios_t *rio, int memory_unit);
 int flist_first_free_rio (rios_t *rio, int memory_unit);
+flist_rio_t *get_flist_rio (rios_t *rio, uint memory_unit, uint file_no);
 
 /* song_management.c */
 int do_upload (rios_t *rio, u_int8_t memory_unit, int addpipe, info_page_t info, int overwrite);
@@ -630,14 +647,16 @@ int update_db_rio (rios_t *rio);
 /* cksum.c */
 u_int32_t crc32_rio (u_int8_t *, size_t);
 
-/* hexdump.c */
-void pretty_print_block (unsigned char *, int, FILE *);
+/* playlist.c */
+int new_playlist_info (info_page_t *newInfo, char *name);
+int playlist_info (info_page_t *newInfo, char *file_name);
+
 
 #ifndef HAVE_BASENAME
 char *basename(char *x);
 #endif
 
-void file_to_me (rio_file_t *);
-void mem_to_me  (rio_mem_t *);
+void file_to_arch (rio_file_t *);
+void mem_to_arch  (rio_mem_t *);
 
 #endif /* _RIO_INTERNAL_H */
