@@ -1,19 +1,19 @@
 /**
- *   (c) 2001-2006 Nathan Hjelm <hjelmn@users.sourceforge.net>
- *   v1.5.2.1 rio.c
+ *   (c) 2001-2016 Nathan Hjelm <hjelmn@users.sourceforge.net>
+ *   v1.5.3 rio.c
  *   
  *   c version of librioutil
  *   all sources are c style gnu (c-set-style in emacs)
  *   
  *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   it under the terms of the GNU Lesser General Public License as published
+ *   by the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
  *   
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ *   GNU Lesser General Public License for more details.
  *   
  *   You should have received a copy of the GNU Library Public License
  *   along with this program; if not, write to the Free Software
@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <inttypes.h>
 
 #include <unistd.h>
 #include <time.h>
@@ -38,34 +39,36 @@
 #endif
 
 #include "rioi.h"
+#include "riolog.h"
 #include "driver.h"
 
 struct player_device_info player_devices[] = {
   /* Rio 600/800/900 and Nike psa[play Use bulk endpoint 2 for read/write */
-  {VENDOR_DIAMOND01, PRODUCT_RIO600 , 0x2, 0x2, RIO600   , 3},
-  {VENDOR_DIAMOND01, PRODUCT_RIO800 , 0x2, 0x2, RIO800   , 3},
-  {VENDOR_DIAMOND01, PRODUCT_PSAPLAY, 0x2, 0x2, PSAPLAY  , 3},
-  {VENDOR_DIAMOND01, PRODUCT_RIO900 , 0x2, 0x2, RIO900   , 3},
+  {VENDOR_DIAMOND01, PRODUCT_RIO600 , 0x2, 0x2, RIO600   , "Rio 600"       , 3},
+  {VENDOR_DIAMOND01, PRODUCT_RIO800 , 0x2, 0x2, RIO800   , "Rio 800"       , 3},
+  {VENDOR_DIAMOND01, PRODUCT_PSAPLAY, 0x2, 0x2, PSAPLAY  , "Nike psa[play" , 3},
+  {VENDOR_DIAMOND01, PRODUCT_RIO900 , 0x2, 0x2, RIO900   , "Rio 900"       , 3},
   /* Rio S-Series Uses bulk endpoint 1 for read and 2 for write */
-  {VENDOR_DIAMOND01, PRODUCT_RIOS30 , 0x1, 0x2, RIOS30   , 4},
-  {VENDOR_DIAMOND01, PRODUCT_RIOS35 , 0x1, 0x2, RIOS35   , 4},
-  {VENDOR_DIAMOND01, PRODUCT_RIOS10 , 0x1, 0x2, RIOS10   , 4},
-  {VENDOR_DIAMOND01, PRODUCT_RIOS50 , 0x1, 0x2, RIOS50   , 4},
-  {VENDOR_DIAMOND01, PRODUCT_FUSE   , 0x1, 0x2, RIOFUSE  , 5},
-  {VENDOR_DIAMOND01, PRODUCT_CHIBA  , 0x1, 0x2, RIOCHIBA , 5},
-  {VENDOR_DIAMOND01, PRODUCT_CALI   , 0x1, 0x2, RIOCALI  , 5},
-  {VENDOR_DIAMOND01, PRODUCT_CALI256, 0x1, 0x2, RIOCALI  , 5},
-  {VENDOR_DIAMOND01, PRODUCT_RIOS11 , 0x1, 0x2, RIOS11   , 4},
+  {VENDOR_DIAMOND01, PRODUCT_RIOS30 , 0x1, 0x2, RIOS30   , "Rio S30S"      , 4},
+  {VENDOR_DIAMOND01, PRODUCT_RIOS35 , 0x1, 0x2, RIOS35   , "Rio S35S"      , 4},
+  {VENDOR_DIAMOND01, PRODUCT_RIOS10 , 0x1, 0x2, RIOS10   , "Rio S10"       , 4},
+  {VENDOR_DIAMOND01, PRODUCT_RIOS50 , 0x1, 0x2, RIOS50   , "Rio S50"       , 4},
+  {VENDOR_DIAMOND01, PRODUCT_FUSE   , 0x1, 0x2, RIOFUSE  , "Rio Fuse"      , 5},
+  {VENDOR_DIAMOND01, PRODUCT_CHIBA  , 0x1, 0x2, RIOCHIBA , "Rio Chiba"     , 5},
+  {VENDOR_DIAMOND01, PRODUCT_CALI   , 0x1, 0x2, RIOCALI  , "Rio Cali"      , 5},
+  {VENDOR_DIAMOND01, PRODUCT_CALI256, 0x1, 0x2, RIOCALI  , "Rio Cali 256"  , 5},
+  {VENDOR_DIAMOND01, PRODUCT_RIOS11 , 0x1, 0x2, RIOS11   , "Rio S11"       , 4},
   /* Rio Riot Uses bulk endpoint 2 for read and 1 for write */
-  {VENDOR_DIAMOND01, PRODUCT_RIORIOT, 0x2, 0x1, RIORIOT  , 3},
-  {VENDOR_DIAMOND01, PRODUCT_NITRUS,  0x1, 0x2, RIONITRUS, 5},
-  {0}
+  {VENDOR_DIAMOND01, PRODUCT_RIORIOT, 0x2, 0x1, RIORIOT  , "Rio Riot"      , 3},
+  {VENDOR_DIAMOND01, PRODUCT_NITRUS,  0x1, 0x2, RIONITRUS, "Rio Nitrus"    , 5},
+  {0,0,0,0,0,NULL,0}
 };
 
 
 /* statically defined functions */
 static int set_time_rio (rios_t *rio);
 static int return_intrn_info_rio(rios_t *rio);
+static void free_info_rio (rios_t *rio);
 
 /* read the supported file types from the rio */
 static int read_ftypes_rio (rios_t *rio) {
@@ -95,20 +98,24 @@ static int read_ftypes_rio (rios_t *rio) {
 int open_rio (rios_t *rio, int number, int debug, int fill_structures) {
   int ret;
 
+  set_debug_out( stderr );
+  set_debug_level( debug );
+
+  debug("open_rio(rio=%x,number=%d,debug=%d,fill_structures=%d)", \
+	rio, number, debug, fill_structures);
+
   if (rio == NULL)
-    return -1;
+    return -EINVAL;
 
   memset(rio, 0, sizeof(rios_t));
   
   rio->debug       = debug;
   rio->log         = stderr;
   
-  rio_log (rio, 0,
-	   "open_rio: creating new rio instance. device: 0x%08x\n", number);
-  
-  if (debug) {
-    rio_log (rio, 0, "open_rio: setting usb driver verbosity level to %i\n",
-	     debug);
+  debug("creating new rio instance. device: 0x%08x", number);
+
+  if (debug > 2) {
+    debug("open_rio: setting usb driver verbosity level to %i", debug);
 
     usb_setdebug(debug);
   }
@@ -117,7 +124,7 @@ int open_rio (rios_t *rio, int number, int debug, int fill_structures) {
   
   /* open the USB device (this calls the underlying driver) */
   if ((ret = usb_open_rio (rio, number)) != 0) {
-    rio_log (rio, ret, "open_rio: could not open a Rio device\n");
+    error("open_rio: could not open a Rio device: %d", ret);
 
     return ret;
   }
@@ -146,7 +153,7 @@ int open_rio (rios_t *rio, int number, int debug, int fill_structures) {
     }
   }
 
-  rio_log (rio, 0, "open_rio: new rio instance created.\n");
+  debug("open_rio: new rio instance created.");
 
   return URIO_SUCCESS;
 }
@@ -172,11 +179,11 @@ static int set_time_rio (rios_t *rio) {
   gettimeofday (&tv, &tz);
   tmp = localtime ((const time_t *)&(tv.tv_sec));
 
-  rio_log (rio, 0, "librioutil/rio.c set_time_rio: Setting device time from system clock: %s\n", asctime(tmp));
+  debug("rio.c set_time_rio: Setting device time from system clock: %s", asctime(tmp));
 
   curr_time = tv.tv_sec - 60 * tz.tz_minuteswest;
   
-  if (tmp->tm_isdst != -1)
+  if (tmp->tm_isdst > 0)
     curr_time += 3600 * tmp->tm_isdst;
 
   ret = send_command_rio (rio, 0x60, 0, 0);
@@ -199,9 +206,9 @@ void close_rio (rios_t *rio) {
   if (try_lock_rio (rio) != 0)
     return;
   
-  rio_log (rio, 0, "close_rio: entering...\n");
+  debug("close_rio: entering...");
 
-  wake_rio (rio);
+  (void)wake_rio (rio);
   
   /* close connection */
   usb_close_rio (rio);
@@ -212,18 +219,18 @@ void close_rio (rios_t *rio) {
 
   unlock_rio (rio);
   
-  rio_log (rio, 0, "close_rio: complete\n");
+  debug("close_rio: complete");
 }
 
-int get_file_info_rio(rios_t *rio, rio_file_t *file,
-		      u_int8_t memory_unit, u_int16_t file_no) {
+int get_file_info_rio(rios_t *rio, rio_file_t *file, u_int8_t memory_unit, u_int16_t file_no) {
   int ret;
-  
-  if (file == NULL)
-    return -1;
 
-  if ((ret = wake_rio(rio)) != URIO_SUCCESS)
-    return ret;
+  debug("get_file_info_rio()");
+
+  if (rio == NULL || file == NULL)
+    return -EINVAL;
+
+  (void)wake_rio(rio);
 
   memset (file, 0, sizeof (rio_file_t));
 
@@ -241,13 +248,14 @@ int get_file_info_rio(rios_t *rio, rio_file_t *file,
       return ret;
 
     /* library handles endianness */
-    file_to_me(file);
+    file_to_arch(file);
     
     /* no file exists with number 0, they are listed from 1 */
     if (file->file_no == 0)
       return -ENOENT;
   } else {
     /* for the RIOT to delete files (does this also work with downloads?) */
+    file->file_no      = file_no;
     file->riot_file_no = file_no;
   }
 
@@ -266,7 +274,7 @@ int generate_mem_list_rio (rios_t *rio) {
 
   mlist_rio_t *list = rio->info.memory;
 
-  rio_log (rio, 0, "create_mem_list_rio: entering...\n");
+  debug("create_mem_list_rio: entering...");
 
   memset(list, 0, sizeof(mlist_rio_t) * MAX_MEM_UNITS);
 
@@ -277,6 +285,8 @@ int generate_mem_list_rio (rios_t *rio) {
     if (ret != URIO_SUCCESS)
       return ret;
 
+    rio->info.total_memory_units = 1;
+
     list[0].size       = memory.size;
     list[0].free       = memory.free;
       
@@ -285,6 +295,8 @@ int generate_mem_list_rio (rios_t *rio) {
     if (ret != URIO_SUCCESS)
       return ret;
   } else {
+    rio->info.total_memory_units = 0;
+
     for (i = 0 ; i < num_mem_units ; i++) {
       ret = get_memory_info_rio (rio, &memory, i);
       
@@ -292,7 +304,9 @@ int generate_mem_list_rio (rios_t *rio) {
 	break; /* not an error */
       else if (ret != URIO_SUCCESS)
 	return ret;
-      
+
+      rio->info.total_memory_units++;
+
       list[i].size       = memory.size;
       list[i].free       = memory.free;
       strncpy(list[i].name, memory.name, 32);
@@ -304,7 +318,7 @@ int generate_mem_list_rio (rios_t *rio) {
     }
   }
 
-  rio_log (rio, 0, "create_mem_list_rio: complete\n");
+  debug("create_mem_list_rio: complete");
 
   return URIO_SUCCESS;
 }
@@ -315,18 +329,17 @@ int get_memory_info_rio(rios_t *rio, rio_mem_t *memory, u_int8_t memory_unit) {
   if (!rio)
     return -1;
   
-  if ((ret = wake_rio(rio)) != URIO_SUCCESS)
-    return ret;
+  (void)wake_rio(rio);
 
-  if (send_command_rio(rio, RIO_MEMRI, memory_unit, 0) != URIO_SUCCESS)
-    return -1;
+  if ((ret = send_command_rio(rio, RIO_MEMRI, memory_unit, 0)) != URIO_SUCCESS)
+    return ret;
 
   /* command was successful, read 256 bytes from Rio */
   if ((ret = read_block_rio(rio, (unsigned char *)memory, 256, RIO_FTS)) != URIO_SUCCESS) 
       return ret;
 
   /* swap to big endian if needed */
-  mem_to_me(memory);
+  mem_to_arch (memory);
   
   /* if requested memory unit is out of range Rio returns 256 bytes of 0's */
   if (memory->size == 0)
@@ -348,14 +361,14 @@ int return_type_rio(rios_t *rio) {
   return ((struct rioutil_usbdevice *)rio->dev)->entry->type;
 }
 
-  /*
-    first generation : Rio300 (Unsupported)
-    second generation: Rio500 (Unsupported)
-    third generation : Rio600, Rio800, Rio900, psa[play, Riot
-    fourth generation: S-Series
-    fith generation  : Fuse, Chiba, Cali
-                       Nitrus, Eigen, Karma (Unsupported)
-  */
+/*
+  first generation : Rio300 (Unsupported)
+  second generation: Rio500 (Unsupported)
+  third generation : Rio600, Rio800, Rio900, psa[play, Riot
+  fourth generation: S-Series
+  fith generation  : Fuse, Chiba, Cali, Nitrus
+                     Eigen, Karma (Unsupported)
+*/
 int return_generation_rio (rios_t *rio) {
   return ((struct rioutil_usbdevice *)rio->dev)->entry->gen;
 }
@@ -364,133 +377,117 @@ float return_version_rio (rios_t *rio) {
   return rio->info.firmware_version;
 }
 
-/*
-  return_intrn_info_rio:
-  BIG function that fills the rio_info structure.
+static int get_device_prefs_rio (rios_t *rio, rio_info_t *infop) {
+  int ret;
+  unsigned char buffer[RIO_MTS];
 
-  renamed from return_info_rio
+  /* iTunes sends this set of commands before RIO_PREFR */
+  ret = send_command_rio(rio, RIO_PREFR, 0, 0);
+  if (ret != URIO_SUCCESS) {
+    error("get_device_prefs_rio: rio did not respond to read preferences command.");
+
+    return ret;
+  }
+
+  ret = read_block_rio (rio, buffer, RIO_MTS, RIO_FTS);
+  if (ret != URIO_SUCCESS) {
+    error("return_info_rio: error reading preference data");
+
+    return ret;
+  }
+
+  if (return_type_rio (rio) != RIORIOT) { /* All but the RIOT */
+    rio_prefs_t prefs;
+
+    /* copy into the prefs structure */
+    memcpy ((void *)&prefs, buffer, RIO_MTS);
+
+    /* Copy the prefs into the info structure */
+    memcpy(infop->name, prefs.name, 17);
+    infop->volume           = prefs.volume;
+    infop->playlist         = prefs.playlist;
+    infop->contrast         = prefs.contrast - 1;
+    infop->sleep_time       = prefs.sleep_time % 5;
+    infop->treble           = prefs.treble;
+    infop->bass             = prefs.bass;
+    infop->eq_state         = prefs.eq_state % 8;
+    infop->repeat_state     = prefs.repeat_state % 4;
+    infop->light_state      = prefs.light_state % 6;
+    infop->random_state     = 0; /* RIOT Only */
+    infop->the_filter_state = 0; /* RIOT Only */
+  } else { /* This is a RIOT */
+    riot_prefs_t riot_prefs;  
+
+    /* copy into the prefs structure */
+    memcpy ((void *)&riot_prefs, buffer, RIO_MTS);
+
+    /* Copy the riot_prefs into the info structure */
+    memcpy(infop->name, riot_prefs.name, 17);
+    infop->volume           = riot_prefs.volume;
+    infop->contrast         = riot_prefs.contrast - 1; /* do we really need the -1 */
+    infop->sleep_time       = riot_prefs.sleep_time;
+    infop->treble           = riot_prefs.treble;
+    infop->bass             = riot_prefs.bass;
+    infop->repeat_state     = riot_prefs.repeat_state % 4; /* Do we really need the mod 4? */
+    infop->light_state      = riot_prefs.light_state;
+    infop->random_state     = riot_prefs.random_state;
+    infop->the_filter_state = riot_prefs.the_filter_state;
+    infop->eq_state         = 0; /* Not on RIOT */
+    infop->playlist         = 0; /* Not on RIOT */
+  }
+
+  return 0;
+}
+
+static int get_device_description_rio (rios_t *rio, rio_info_t *infop) {
+  int ret;
+  unsigned char desc[256];
+
+  ret = send_command_rio(rio, RIO_DESCP, 0, 0);
+  if (ret != URIO_SUCCESS)
+    return ret;
+
+  ret = read_block_rio(rio, desc, 256, RIO_FTS);
+  if (ret != URIO_SUCCESS)
+    return ret;
+
+  infop->firmware_version = desc[5] + (0.1) * (desc[4] >> 4) + (0.01) * (desc[4] & 0xf);
+  memmove (infop->serial_number, &desc[0x60], 16);
+
+  return 0;
+}
+
+/*
+  return_intrn_info_rio: fill the rio_info structure.
 */
 static int return_intrn_info_rio(rios_t *rio) {
-  rio_info_t *info = &rio->info;
-  rio_prefs_t prefs;
-  riot_prefs_t riot_prefs;
-  
-  unsigned char desc[256];
-  unsigned char cmd;
-
   int ret;
-  int i;
   
   if ((ret = try_lock_rio (rio)) != 0)
     return ret;
 
-  memset (info, 0, sizeof (rio_info_t));
+  (void)wake_rio (rio);
 
-  /* 
-   * Send the initialize set of commands to the RIO
-   */
-  ret = send_command_rio(rio, 0x66, 0, 0);
-  if (ret != URIO_SUCCESS) {
-     rio_log (rio, ret, "return_info_rio: Error sending command\n");
-    UNLOCK(ret);
-  }
-
-  ret = send_command_rio(rio, 0x65, 0, 0);
-  if (ret != URIO_SUCCESS) {
-    rio_log (rio, ret, "return_info_rio: Error sending command\n");
-    UNLOCK(ret);
-  }
-
-  /*
-    retrieve non-changable values
-  */
-  cmd = RIO_DESCP;
-  ret = send_command_rio(rio, cmd, 0, 0);
-  if (ret != URIO_SUCCESS) {
-    rio_log (rio, ret, "return_info_rio: error sending command.\n");
-    UNLOCK(ret);
-  }
-
-  ret = read_block_rio(rio, desc, 256, RIO_FTS);
-  if (ret != URIO_SUCCESS) {
-    rio_log (rio, ret, "return_info_rio: Error reading device info\n", cmd);
-
-    UNLOCK(ret);
-  }
-
-  info->firmware_version = (desc[5] + (0.1) * (desc[4] >> 4)
-			    + (0.01) * (desc[4] & 0xf));
-  memmove (info->serial_number, &desc[0x60], 16);
-
-  /*
-   * this is where we set which structure to use to fill the
-   * prefs
-   */
+  memset (&rio->info, 0, sizeof (rio_info_t));
   
-  if ((ret = generate_mem_list_rio(rio)) != URIO_SUCCESS) 
-    return ret;
+  /* retrieve serial number and firmware version */
+  ret = get_device_description_rio (rio, &rio->info);
+  if (ret < 0) {
+    error("rio.c return_intrn_info_rio: error reading device description.");
 
-  /*
-    retrieve changeable values
-  */
-  /* iTunes sends this set of commands before RIO_PREFR */
-  cmd = RIO_PREFR;
-  if ((ret = send_command_rio(rio, cmd, 0, 0)) == URIO_SUCCESS) {
-    rio_log (rio, ret, "return_info_rio: Preference read command successful\n");
+    UNLOCK(ret);
+  }
 
-    if (return_type_rio (rio) != RIORIOT) { /* All but the RIOT */
-      
-      /* Read a block into the prefs structure */	    
-      ret = read_block_rio(rio, (unsigned char *)&prefs, RIO_MTS, RIO_FTS);
-      if (ret != URIO_SUCCESS) {
-        rio_log (rio, ret, "return_info_rio: Error reading data after command 0x%x\n", cmd);
-        UNLOCK(ret);
-      }
-
-      /* Copy the prefs into the info structure */
-      memcpy(info->name, prefs.name, 17);
-      info->volume           = prefs.volume;
-      info->playlist         = prefs.playlist;
-      info->contrast         = prefs.contrast - 1;
-      info->sleep_time       = prefs.sleep_time % 5;
-      info->treble           = prefs.treble;
-      info->bass             = prefs.bass;
-      info->eq_state         = prefs.eq_state % 8;
-      info->repeat_state     = prefs.repeat_state % 4;
-      info->light_state      = prefs.light_state % 6;
-      info->random_state     = 0; /* RIOT Only */
-      info->the_filter_state = 0; /* RIOT Only */
-
-    } else { /* This is a RIOT */
-      /* Read a block into the riot_prefs structure */
-      ret = read_block_rio(rio, (unsigned char *)&riot_prefs, RIO_MTS, RIO_FTS);
-      if (ret != URIO_SUCCESS) {
-        rio_log (rio, ret, "return_info_rio: Error reading data from RIOT after command 0x%x\n",cmd);
-	UNLOCK(ret);
-      }
-
-      /* Copy the riot_prefs into the info structure */
-      memcpy(info->name, riot_prefs.name, 17);
-      info->volume           = riot_prefs.volume;
-      info->contrast         = riot_prefs.contrast - 1; /* do we really need the -1 */
-      info->sleep_time       = riot_prefs.sleep_time;
-      info->treble           = riot_prefs.treble;
-      info->bass             = riot_prefs.bass;
-      info->repeat_state     = riot_prefs.repeat_state % 4; /* Do we really need the mod 4? */
-      info->light_state      = riot_prefs.light_state;
-      info->random_state     = riot_prefs.random_state;
-      info->the_filter_state = riot_prefs.the_filter_state;
-      info->eq_state         = 0; /* Not on RIOT */
-      info->playlist         = 0; /* Not on RIOT */
-    }
-  } else /* Failed the read */ 
-      rio_log (rio, -1, "return_info_rio: Rio did not respond to Preference read command.\n");
+  /* retrieve user preferences */
+  (void)get_device_prefs_rio (rio, &rio->info);
      
-  /*
-    memory
-  */
-  for (i = 0 ; info->memory[i].size && i < MAX_MEM_UNITS ; i++)
-    info->total_memory_units++;
+  /* generate internal file ane memory lists */
+  ret = generate_mem_list_rio(rio);
+  if (ret != URIO_SUCCESS) {
+    error("rio.c return_intrn_info_rio: could not generate memory/file listing");
+
+    UNLOCK(ret);
+  }
 
   UNLOCK(URIO_SUCCESS);
 }
@@ -512,7 +509,9 @@ static void sane_info_copy (rio_info_t *info, rio_prefs_t *prefs);
 int set_info_rio(rios_t *rio, rio_info_t *info) {
   rio_prefs_t pref_buf;
   int ret;
-  unsigned char cmd;
+
+  if (rio == NULL)
+    return -EINVAL;
 
   if ((ret = try_lock_rio (rio)) != 0)
     return ret;
@@ -521,45 +520,58 @@ int set_info_rio(rios_t *rio, rio_info_t *info) {
   if (info == NULL)
     return -1;
 
-  cmd = RIO_PREFR;
-  if ((ret = send_command_rio(rio, cmd, 0, 0)) != 0) {
-    rio_log (rio, ret, "set_info_rio: Error sending command\n");
+  (void)wake_rio (rio);
+  
+  ret = send_command_rio(rio, RIO_PREFR, 0, 0);
+  if (ret != URIO_SUCCESS) {
+    error("set_info_rio: error sending read preferences command: %d", ret);
 
     UNLOCK(ret);
   }
   
   ret = read_block_rio(rio, (unsigned char *)&pref_buf, RIO_MTS, RIO_FTS);
   if (ret != URIO_SUCCESS) {
-    rio_log (rio, ret, "Error reading data after command 0x%x\n", cmd);
-    
+    error("set_info_rio: error reading preference data: %d", ret);
+
     UNLOCK(ret);
   }
   
   sane_info_copy (info, &pref_buf);
   
-  if ((ret = wake_rio(rio)) != URIO_SUCCESS)
-    UNLOCK(ret);
-
-  cmd = RIO_PREFS;
-  if ((ret = send_command_rio(rio, cmd, 0, 0)) != 0) {
-    rio_log (rio, ret, "set_info_rio: Error sending command\n");
+  ret = send_command_rio(rio, RIO_PREFS, 0, 0);
+  if (ret != URIO_SUCCESS) {
+    error("set_info_rio: error sending set preferences command: %d", ret);
 
     UNLOCK(ret);
   }
 
   ret = read_block_rio(rio, NULL, 64, RIO_FTS);
   if (ret != URIO_SUCCESS) {
-    rio_log (rio, ret, "set_info_rio: error reading data after command 0x%x\n", cmd);
-    
+    error("set_info_rio: error reading from device: %d", ret);
+
     UNLOCK(ret);
   }
 
   if ((ret = write_block_rio(rio, (unsigned char *)&pref_buf, RIO_MTS, NULL)) != URIO_SUCCESS)
-    rio_log (rio, ret, "set_info_rio: error writing preferences\n");
+    error("set_info_rio: error writing preferences: %d", ret);
 
   UNLOCK(ret);
 }
 
+int set_name_rio (rios_t *rio, char *name) {
+  int ret;
+
+  if (rio == NULL || name == NULL)
+    return -EINVAL;
+
+  if (rio->info.memory[0].size == 0 && (ret = return_intrn_info_rio (rio)) != URIO_SUCCESS)
+    return ret;
+
+  strncpy (rio->info.name, name, 16);
+
+  return set_info_rio (rio, &rio->info);
+}
+
 /*
   sane_info_copy:
   Make sure all values of info are sane and put them into prefs.
@@ -592,12 +604,12 @@ int format_mem_rio (rios_t *rio, u_int8_t memory_unit) {
   if ((ret = try_lock_rio (rio)) != 0)
     return ret;
 
-  rio_log (rio, 0, "librioutil/rio.c format_mem_rio: erasing memory unit %i\n", memory_unit);
+  debug("rio.c format_mem_rio: erasing memory unit %i", memory_unit);
 
   /* don't need to call wake_rio here */
 
   if (rio->progress)
-    rio->progress (0, 100, rio->progress_ptr);
+    rio->progress (0, 1, rio->progress_ptr);
 
   if ((ret = send_command_rio(rio, RIO_FORMT, memory_unit, 0)) != URIO_SUCCESS)
     UNLOCK(ret);
@@ -608,7 +620,7 @@ int format_mem_rio (rios_t *rio, u_int8_t memory_unit) {
 
     /* newer players (Fuse, Chiba, Cali) return their progress */
     if (strstr((char *)rio->buffer, "SRIOPR") != NULL) {
-      sscanf ((char *)rio->buffer, "SRIOPR%02d", &pd);
+      sscanf ((char *)rio->buffer, "SRIOPR%" SCNu32, &pd);
 
       if (rio->progress)
 	rio->progress (pd, 100, rio->progress_ptr);
@@ -616,16 +628,16 @@ int format_mem_rio (rios_t *rio, u_int8_t memory_unit) {
       /* format operation completed successfully */
       break;
     } else {
-      rio_log (rio, -1, "librioutil/rio.c format_mem_rio: erase failed\n");
+      error("librioutil/rio.c format_mem_rio: erase failed");
 
       UNLOCK(-1);
     }
   }
 
   if (rio->progress)
-    rio->progress (100, 100, rio->progress_ptr);
+    rio->progress (1, 1, rio->progress_ptr);
 
-  rio_log (rio, 0, "librioutil/rio.c format_mem_rio: erase complete\n");
+  debug("rio.c format_mem_rio: erase complete");
 
   UNLOCK(URIO_SUCCESS);
 }
@@ -636,7 +648,7 @@ int format_mem_rio (rios_t *rio, u_int8_t memory_unit) {
   Update the firmware on a Rio. Function supports all rioutil supported players.
 */
 int update_rio (rios_t *rio, char *file_name) {
-  rio_log (rio, 0, "update_rio: function depricated. use firmware_upgrade_rio instead.\n");
+  warning("update_rio: function deprecated. use firmware_upgrade_rio instead.");
 
   return firmware_upgrade_rio (rio, file_name);
 }
@@ -655,21 +667,15 @@ int firmware_upgrade_rio (rios_t *rio, char *file_name) {
   int ret, pg;
   int player_generation;
 
-  if (file_name == NULL || stat(file_name, &statinfo) < 0)
+  if (rio == NULL || file_name == NULL || stat(file_name, &statinfo) < 0)
     return -EINVAL;
 
   size = statinfo.st_size;
 
-
-  rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: updating firmware of generation %d rio...\n",
-	   player_generation);
-
-  
-  if ((ret = wake_rio(rio)) != URIO_SUCCESS)
-    UNLOCK(ret);
+  (void)wake_rio(rio);
 
   /* some upgrades require that the memory unit be erased */
-  rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: formatting internal memory\n");
+  debug("rio.c firmware_upgrade_rio: formatting internal memory");
   if ((ret = format_mem_rio (rio, 0)) != URIO_SUCCESS)
     UNLOCK(ret);
 
@@ -678,34 +684,36 @@ int firmware_upgrade_rio (rios_t *rio, char *file_name) {
 
   player_generation = return_generation_rio (rio);
 
+  debug ("rio.c firmware_upgrade_rio: updating firmware of generation %d rio...",
+         player_generation);
 
   /* try to open the firmware file */
   if ((firm_fd = open(file_name, O_RDONLY)) < 0)
     UNLOCK(errno);
 
   /* it is not necessary to check the .lok file as the player will reject bad input */
-  rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: sending firmware update device command...\n");
+  debug("rio.c firmware_upgrade_rio: sending firmware update device command...");
 
   if ((ret = send_command_rio(rio, RIO_UPDAT, 0x1, 0)) != URIO_SUCCESS) {
-    rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: device did not respond to command.\n");
+    error("rio.c firmware_upgrade_rio: device did not respond to command.");
 
     close (firm_fd);
     UNLOCK(ret);
   }
 
   if ((ret = read_block_rio(rio, rio->buffer, 64, RIO_FTS)) != URIO_SUCCESS) {
-    rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: device did not respond as expected.\n");
+    error("rio.c firmware_upgrade_rio: device did not respond as expected.");
     
     close (firm_fd);
     UNLOCK(ret);
   }
   
-  rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: device acknowleged command.\n");
+  debug("rio.c firmware_upgrade_rio: device acknowleged command.");
 
   if (player_generation > 3)
-    rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: erasing...\n");
+    debug("librioutil/rio.c firmware_upgrade_rio: erasing...");
   else
-    rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: writing firmware...\n");
+    debug("librioutil/rio.c firmware_upgrade_rio: writing firmware...");
 
   /* send the size of the firmware data */
   memset(rio->buffer, 0, 64);
@@ -716,6 +724,10 @@ int firmware_upgrade_rio (rios_t *rio, char *file_name) {
   if ((ret = write_block_rio(rio, rio->buffer, 64, NULL)) != URIO_SUCCESS)
     UNLOCK(ret);
 
+  /* initialize progress callback */
+  if (rio->progress != NULL)
+    rio->progress (0, 1, rio->progress_ptr);
+
   /* on newer players the first write apparently erases the device (data is still sent) */
   for (blocks = size / blocksize, x = 0 ; x < blocks; x++) {
     /* read in a chunk of file */
@@ -730,7 +742,7 @@ int firmware_upgrade_rio (rios_t *rio, char *file_name) {
 	  rio->progress (pg, 200, rio->progress_ptr);
       } else if (strstr ((char *)rio->buffer, "SRIODONE") != NULL) {
 	if (rio->progress != NULL)
-	  rio->progress (100, 100, rio->progress_ptr);
+	  rio->progress (1, 1, rio->progress_ptr);
 
 	close (firm_fd);
 	return URIO_SUCCESS;
@@ -749,7 +761,7 @@ int firmware_upgrade_rio (rios_t *rio, char *file_name) {
 
   if (player_generation > 3) {
     /* if this is a newer player the update is not quite done */
-    rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: writing firmware...\n");
+    debug("librioutil/rio.c firmware_upgrade_rio: writing firmware...");
 
     /* it takes a moment before the rio is ready to continue */
     usleep (1000);
@@ -780,11 +792,11 @@ int firmware_upgrade_rio (rios_t *rio, char *file_name) {
 
   /* make sure the progress bar reaches 100% */
   if (rio->progress != NULL)
-    rio->progress (blocks, blocks, rio->progress_ptr);
+    rio->progress (1, 1, rio->progress_ptr);
 
   close(firm_fd);
 
-  rio_log (rio, 0, "librioutil/rio.c firmware_upgrade_rio: firmware update complete\n");
+  debug("rio.c firmware_upgrade_rio: firmware update complete");
 
   UNLOCK(URIO_SUCCESS);
 }
@@ -811,7 +823,7 @@ int wake_rio (rios_t *rio) {
 }
 
 /* frees the info ptr in rios_t structure */
-void free_info_rio (rios_t *rio) {
+static void free_info_rio (rios_t *rio) {
   int i;
   flist_rio_t *tmp, *ntmp;
   
@@ -822,7 +834,6 @@ void free_info_rio (rios_t *rio) {
     }
 }
 
-/* New Functions -- Aug 8 2001 */
 /*
   update_info_rio:
 
@@ -843,9 +854,9 @@ int update_info_rio (rios_t *rio) {
 
   returns to total number of memory units an instance has.
 */
-int return_mem_units_rio (rios_t *rio) {
+uint return_mem_units_rio (rios_t *rio) {
   if (rio == NULL)
-    return -EINVAL;
+    return 0;
   
   return rio->info.total_memory_units;
 }
@@ -860,8 +871,7 @@ int return_free_mem_rio (rios_t *rio, u_int8_t memory_unit) {
     return -EINVAL;
   
   if (memory_unit >= MAX_MEM_UNITS) {
-    rio_log (rio, -2, "return_free_mem_rio: memory unit %02x out of range.\n",
-	     memory_unit);
+    error("return_free_mem_rio: memory unit %02x out of range.", memory_unit);
     return -2;
   }
 
@@ -878,8 +888,7 @@ int return_used_mem_rio (rios_t *rio, u_int8_t memory_unit) {
     return -EINVAL;
 
   if (memory_unit >= MAX_MEM_UNITS) {
-    rio_log (rio, -2, "return_used_mem_rio: memory unit %02x out of range.\n",
-	     memory_unit);
+    error("return_used_mem_rio: memory unit %02x out of range.", memory_unit);
     return -2;
   }
   
@@ -896,8 +905,7 @@ int return_total_mem_rio (rios_t *rio, u_int8_t memory_unit) {
     return -EINVAL;
 
   if (memory_unit >= MAX_MEM_UNITS) {
-    rio_log (rio, -2, "return_total_mem_rio: memory unit %02x out of range.\n",
-	     memory_unit);
+    error("return_total_mem_rio: memory unit %02x out of range.", memory_unit);
     return -2;
   }
   
@@ -918,8 +926,7 @@ char *return_file_name_rio(rios_t *rio, u_int32_t song_id,
     return NULL;
   
   if (memory_unit >= MAX_MEM_UNITS) {
-    rio_log (rio, -2, "return_file_name_rio: memory unit %02x out of range.\n",
-	     memory_unit);
+    error("return_file_name_rio: memory unit %02x out of range.", memory_unit);
     return NULL;
   }
   
@@ -944,9 +951,7 @@ int return_file_size_rio(rios_t *rio, u_int32_t song_id, u_int8_t memory_unit) {
     return -1;
   
   if (memory_unit >= MAX_MEM_UNITS) {
-    rio_log (rio, -2,
-	     "return_file_size_rio: memory unit %02x out of range.\n",
-	     memory_unit);
+    error("return_file_size_rio: memory unit %02x out of range.", memory_unit);
     return -2;
   }
   
@@ -971,9 +976,7 @@ int return_num_files_rio (rios_t *rio, u_int8_t memory_unit) {
     return -EINVAL;
   
   if (memory_unit >= MAX_MEM_UNITS) {
-    rio_log (rio, -2,
-	     "return_num_files_rio: memory unit %02x out of range.\n",
-	     memory_unit);
+    error("return_num_files_rio: memory unit %02x out of range.", memory_unit);
     return -2;
   }
   
@@ -990,8 +993,7 @@ int return_time_rio (rios_t *rio, u_int8_t memory_unit) {
     return -EINVAL;
   
   if (memory_unit >= MAX_MEM_UNITS) {
-    rio_log (rio, -2, "return_time_rio: memory unit %02x out of range.\n",
-	     memory_unit);
+    error("return_time_rio: memory unit %02x out of range.", memory_unit);
     return -2;
   }
   
@@ -1004,7 +1006,7 @@ int return_time_rio (rios_t *rio, u_int8_t memory_unit) {
 rio_info_t *return_info_rio (rios_t *rio) {
   rio_info_t *new_info = NULL;
 
-  rio_log (rio, 0, "return_info_rio: function depricated. use get_info_rio instead.\n");
+  warning("return_info_rio: function deprecated. use get_info_rio instead.");
 
   get_info_rio (rio, &new_info);
   
@@ -1063,7 +1065,7 @@ int try_lock_rio (rios_t *rio) {
     return -EINVAL;
 
   if (rio->lock != 0) {
-    rio_log (rio, -EBUSY, "librioutil/rio.c try_lock_rio: rio is being used by another thread.\n");
+    error("librioutil/rio.c try_lock_rio: rio is being used by another thread.");
 
     return -EBUSY;
   }
