@@ -1,16 +1,16 @@
- /**
- *   (c) 2001-2006 Nathan Hjelm <hjelmn@users.sourceforge.net>
- *   v2.0.1 song_managment.c
+/**
+ *   (c) 2001-2016 Nathan Hjelm <hjelmn@users.sourceforge.net>
+ *   v2.5.1 song_managment.c
  *
  *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   it under the terms of the GNU Lesser General Public License as published
+ *   by the Free Software Foundation; either version 2 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ *   GNU Lesser General Public License for more details.
  *
  *   You should have received a copy of the GNU Library Public License
  *   along with this program; if not, write to the Free Software
@@ -23,58 +23,60 @@
 #include <time.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <limits.h>
 
 #include <sys/stat.h>
 
 #include "rioi.h"
+#include "riolog.h"
 
 #if defined(HAVE_LIBGEN_H)
 #include <libgen.h>
 #endif
 
+#if !defined(PATH_MAX)
+#define PATH_MAX 1024
+#endif
+
 static int init_new_upload_rio (rios_t *rio, u_int8_t memory_unit);
 static int init_overwrite_rio (rios_t *rio, u_int8_t memory_unit);
 static int complete_upload_rio (rios_t *rio, u_int8_t memory_unit, info_page_t info);
 static int bulk_upload_rio (rios_t *rio, info_page_t info, int addpipe);
-static int delete_dummy_hdr (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno);
-static int upload_dummy_hdr (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno);
+static int upload_dummy_hdr (rios_t *rio, u_int8_t memory_unit, rio_file_t *filexp);
 
 /* the guts of any upload */
 int do_upload (rios_t *rio, u_int8_t memory_unit, int addpipe, info_page_t info, int overwrite) {
   int error;
 
-  rio_log (rio, 0, "do_upload: entering\n");
+  debug("librioutil/song_management.c do_upload: entering");
 
   /* check if there the device has sufficient space for the file */
-  if (overwrite == 0)
+  if (overwrite == 0) {
     if (FREE_SPACE(memory_unit) < (info.data->size - info.skip)/1024) {
       free (info.data);
       
       return -ENOSPC;
     }
     
-  if (overwrite == 0) {
     if ((error = init_new_upload_rio(rio, memory_unit)) != URIO_SUCCESS) {
-      rio_log (rio, error, "init_upload_rio error\n");
+      error("librioutil/song_management.c do_upload: error in init_upload_rio");
       return error;
     }
   } else { 
     if ((error = init_overwrite_rio(rio, memory_unit)) != URIO_SUCCESS) {
-      rio_log (rio, error, "init_upload_rio error\n");
+      error("librioutil/song_management.c do_upload: error in init_overwrite_rio");
       return error;
     }
   }
 
   if ((error = bulk_upload_rio(rio, info, addpipe)) != URIO_SUCCESS) {
-    rio_log (rio, error, "bulk_upload_rio error\n");
+    error("librioutil/song_management.c do_upload: error in bulk_upload_rio");
     abort_transfer_rio(rio);
     return error;
   }
-  
-  close (addpipe);
 
   if ((error = complete_upload_rio(rio, memory_unit, info))!= URIO_SUCCESS) {
-    rio_log (rio, error, "complete_upload_rio error\n");
+    error("librioutil/song_management.c do_upload: error in complete_upload_rio");
     abort_transfer_rio(rio);
     return error;
   }
@@ -87,7 +89,7 @@ int do_upload (rios_t *rio, u_int8_t memory_unit, int addpipe, info_page_t info,
   if (info.data->type == TYPE_MP3)
     update_db_rio (rio);
 
-  rio_log (rio, 0, "do_upload: complete\n");
+  debug("librioutil/song_management.c do_upload: complete");
 
   return URIO_SUCCESS;
 }
@@ -109,8 +111,8 @@ int do_upload (rios_t *rio, u_int8_t memory_unit, int addpipe, info_page_t info,
       - URIO_SUCCESS if the file was uploaded.
       - < 0 if an error occured.
 */
-int add_song_rio (rios_t *rio, u_int8_t memory_unit, char *file_name, char *artist,
-		  char *title, char *album) {
+int add_song_rio (rios_t *rio, u_int8_t memory_unit, char *file_name,
+                  const char *artist, const char *title, const char *album) {
   info_page_t song_info;
   int error;
   int addpipe;
@@ -123,7 +125,7 @@ int add_song_rio (rios_t *rio, u_int8_t memory_unit, char *file_name, char *arti
   if (memory_unit >= rio->info.total_memory_units)
     return -1;
 
-  rio_log (rio, 0, "add_song_rio: entering...\n");
+  debug("add_song_rio: entering...");
   
   if (stat(file_name, &statinfo) < 0)
     return -ENOENT;
@@ -137,19 +139,19 @@ int add_song_rio (rios_t *rio, u_int8_t memory_unit, char *file_name, char *arti
   tmp = strdup (file_name);
   tmp2 = basename(tmp);
   
-  strncpy((char *)song_info.data->name , tmp2, 63);
+  strncpy((char *)song_info.data->name, tmp2, 63);
   
   free (tmp);
 
   /* check for file types by extension */
-  tmp = file_name + strlen(file_name) - 3;
+  tmp = file_name + strlen(file_name) - 4;
 
-  if (strspn(tmp, "mMpP3") == 3) {
+  if (strcasecmp(tmp, ".mp3") == 0) {
     error = mp3_info(&song_info, file_name);
   
     /* just in case one of the info funcs failed */
     if (error != 0) {
-      rio_log (rio, error, "Error getting song info.\n");
+      error("Error getting song info.");
     
       return error;
     }
@@ -166,12 +168,12 @@ int add_song_rio (rios_t *rio, u_int8_t memory_unit, char *file_name, char *arti
   
     if (album)
       sprintf(song_info.data->album, album, 63);
-  } else if (strstr(file_name, ".lst") == NULL && strstr (file_name, ".m3u") == NULL) {
-    if ((error = downloadable_info(&song_info, file_name)) != 0)
-      UNLOCK(error);
-  } else {
+  } else if (strcasecmp (file_name, ".lst") == 0 || strcasecmp (file_name, ".m3u") == 0) {
     if ((error = playlist_info(&song_info, file_name)) != 0)
       return error;
+  } else {
+    if ((error = downloadable_info(&song_info, file_name)) != 0)
+      UNLOCK(error);
   }
 
   /* upload the file */
@@ -179,7 +181,7 @@ int add_song_rio (rios_t *rio, u_int8_t memory_unit, char *file_name, char *arti
   if (addpipe < 0)
     UNLOCK(-errno);
 
-  rio_log (rio, 0, "add_song_rio: file opened and ready to send to rio.\n");
+  debug("add_song_rio: file opened and ready to send to rio.");
 
   if ((error = do_upload (rio, memory_unit, addpipe, song_info, 0)) != URIO_SUCCESS) {
     free(song_info.data);
@@ -193,57 +195,55 @@ int add_song_rio (rios_t *rio, u_int8_t memory_unit, char *file_name, char *arti
 
   free(song_info.data);
   
-  rio_log (rio, 0, "add_song_rio: complete\n");
+  debug("add_song_rio: complete");
 
   UNLOCK(URIO_SUCCESS);
 }
 
-int overwrite_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno, char *filename) {
-  file_list *tmp;
+int overwrite_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t file_num, char *filename) {
   info_page_t song_info;  
   rio_file_t file;
-  int ret;
-  int addpipe;
+  int ret, addpipe, file_id;
   
   struct stat statinfo;
 
+  if (rio == NULL || memory_unit > 1 || filename == NULL)
+    return -EINVAL;
+
+  debug("overwrite_file_rio: entering");
+
+  if (stat (filename, &statinfo) < 0) {
+    error("overwrite_file_rio: could not stat %s", filename);
+
+    return -errno;
+  }
+
   if ((ret = try_lock_rio (rio)) != 0)
     return ret;
   
-  rio_log (rio, 0, "overwrite_file_rio: entering\n");
-
-  if (stat (filename, &statinfo) < 0) {
-    rio_log (rio, 0, "overwrite_file_rio: could not stat %s\n", filename);
-    UNLOCK(-errno);
-  }
-
-  if ((ret = wake_rio(rio)) != URIO_SUCCESS)
-    UNLOCK(ret);
+  (void)wake_rio(rio);
   
   /* hopefully this list is up to date */
-  for (tmp = rio->info.memory[memory_unit].files ; tmp ; tmp = tmp->next)
-    if (tmp->num == fileno)
-      break;
-  
-  /* not really an error if the file doesnt exist */
-  if (!tmp) {
-    rio_log (rio, 0, "overwrite_file_rio: file not found %i on %i\n", memory_unit, fileno);
-    UNLOCK(-1);
+  file_id = flist_get_file_id_rio (rio, memory_unit, file_num);
+  if (file_id < 0) {
+    error("librioutility/song_management.c overwrite_file_rio: file not found.");
+
+    UNLOCK(file_id);
   }
 
-  if (get_file_info_rio(rio, &file, memory_unit, tmp->inum) != URIO_SUCCESS)
+  if (get_file_info_rio(rio, &file, memory_unit, file_id) != URIO_SUCCESS)
     UNLOCK(-1);
 
   file.size = statinfo.st_size;
   song_info.data = &file;
     
   if ((addpipe = open(filename, O_RDONLY)) == -1) {
-    rio_log (rio, errno, "overwrite_file_rio: open failed\n");
-    return -1;
+      error("overwrite_file_rio: open failed: %d", errno);
+    UNLOCK(-1);
   }
   
   if ((ret = do_upload (rio, 0, addpipe, song_info, 1)) != URIO_SUCCESS) {
-    rio_log (rio, 0, "overwrite_file_rio: do_upload failed\n");
+    error("overwrite_file_rio: do_upload failed");
     close (addpipe);
     
     UNLOCK(ret);
@@ -251,95 +251,33 @@ int overwrite_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno, cha
   
   close (addpipe);
   
-  rio_log (rio, 0, "overwrite_file_rio: complete\n");
-  
-  UNLOCK(URIO_SUCCESS);
-}
-
-int upload_from_pipe_rio (rios_t *rio, u_int8_t memory_unit, int addpipe, char *name, char *artist,
-			  char *album, char *title, int mp3, int bitrate, int samplerate) {
-  info_page_t song_info;
-  int error;
- 
-  if (name == NULL || addpipe < 0 || memory_unit >= rio->info.total_memory_units)
-    return -EINVAL;
- 
-  if ((error = try_lock_rio (rio)) != 0)
-    return error;
-
-  if ((song_info.data = (rio_file_t *) calloc (sizeof(rio_file_t), 1)) == NULL) {
-    perror ("upload_from_pipe:");
-
-    UNLOCK(-errno);
-  }
-
-  rio_log (rio, 0, "Adding from pipe %i...\n", addpipe);
-
-  /* copy any user-suplied data*/
-  sprintf(song_info.data->name, name, 63);
-
-  if (artist)
-    sprintf(song_info.data->artist, artist, 63);
-  
-  if (title)
-    sprintf(song_info.data->title, title, 63);
-  
-  if (album)
-    sprintf(song_info.data->album, album, 63);
-
-  if (mp3) {
-    song_info.data->bit_rate = bitrate;
-    song_info.data->sample_rate = samplerate;
-
-    song_info.data->mod_date = time(NULL);
-    song_info.data->bits = 0x10000b11;
-    song_info.data->type = TYPE_MP3;
-    song_info.data->foo4 = 0x00020000;
-  }
-
-  if ((error = do_upload (rio, memory_unit, addpipe, song_info, 0)) != URIO_SUCCESS) {
-    free(song_info.data);
-
-    UNLOCK(error);
-  }
-
-  free(song_info.data);
+  debug("overwrite_file_rio: complete");
   
   UNLOCK(URIO_SUCCESS);
 }
 
 /*
-  init_upload_rio:
-    Send the write command and read ready and first SRIODATA.
-
-    ** Change 7-27-2001 **
-    Backpacks should work now!
+  init_upload_rio: sends write command to the device.
 */
 static int init_upload_rio (rios_t *rio, u_int8_t memory_unit, u_int8_t upload_command) {
   int ret;
 
-  rio_log (rio, 0, "init_upload_rio: entering\n");
+  debug("librioutil/song_management.c init_upload_rio: entering");
 
-  if ((ret = wake_rio(rio)) != URIO_SUCCESS)
-    return ret;
+  (void)wake_rio(rio);
 
   if ((ret = send_command_rio(rio, upload_command, memory_unit, 0)) != URIO_SUCCESS)
     return ret;
   
   read_block_rio(rio, NULL, 64, 64);
-  
   if (strncmp((char *)rio->buffer, "SRIORDY", 7) != 0)
     return -EBUSY;
   
-  if ((int)rio->cmd_buffer[0])
-    read_block_rio(rio, NULL, 64, 64);
-  else
-    return -1;
-  
+  read_block_rio(rio, NULL, 64, 64);
   if (strncmp((char *)rio->buffer, "SRIODATA", 8) != 0)
     return -EBUSY;
   
-  rio_log (rio, 0, "init_upload_rio: finished\n");
+  debug("librioutil/song_management.c init_upload_rio: finished");
 
   return URIO_SUCCESS;
 }
@@ -360,19 +298,20 @@ static int bulk_upload_rio(rios_t *rio, info_page_t info, int addpipe) {
   unsigned char file_buffer[2 * RIO_FTS];
   size_t write_size;
   long int copied = 0, amount;
-  
   int ret;
 
-  if (return_type_rio (rio) == RIONITRUS)
-    write_size = 2 * RIO_FTS;
-  else
-    write_size = RIO_FTS;
+  debug("librioutil/song_management.c bulk_upload_rio: entering");
+  debug("librioutil/song_management.c bulk_upload_rio: skipping %d bytes of input",
+	   info.skip);
+
+  write_size = (return_type_rio (rio) == RIONITRUS) ? (2 * RIO_FTS) : RIO_FTS;
   
-  rio_log (rio, 0, "bulk_upload_rio: entering\n");
-  rio_log (rio, 0, "Skipping %08x bytes of input\n", info.skip);
   lseek(addpipe, info.skip, SEEK_SET);
   memset (file_buffer, 0, write_size);
-  
+
+  if (rio->progress != NULL)
+    rio->progress(0, 1, rio->progress_ptr);
+
   while ((amount = read (addpipe, file_buffer, write_size)) != 0) {
     /* if we dont know the size we dont know how close we are to finishing */
     if (info.data->size && rio->progress != NULL)
@@ -385,18 +324,21 @@ static int bulk_upload_rio(rios_t *rio, info_page_t info, int addpipe) {
     copied += amount;
   }
 
-  rio_log (rio, 0, "Read in %08x bytes from file. File size is %08x\n", copied, info.data->size);
-
-  if (info.data->size == -1) {
+  if (info.data->size == 0) {
     info.data->size = copied;
 
     /* the time value should also be set, but there is no good way to do so */
+    if (info.data->bit_rate)
+      info.data->time = copied/(info.data->bit_rate * 1000);
   }
   
+  debug("librioutil/song_management.c bulk_upload_rio: sent %d/%d bytes to player",
+	   copied, info.data->size);
+
   if (rio->progress != NULL)
     rio->progress(1, 1, rio->progress_ptr);
 
-  rio_log (rio, 0, "bulk_upload_rio: finished\n");
+  debug("librioutil/song_management.c bulk_upload_rio: finished");
 
   return URIO_SUCCESS;
 }
@@ -423,7 +365,7 @@ static int str_cmp (char *str1, char *str2) {
   if (str2l == 0)
     return -1;
 
-  /* Ignore leading "the " in strings */
+  /* ignore leading "the " in strings */
   if (strlen (str1) > 4 && strncasecmp (str1, "the ", 4) == 0)
     str1p = &str1[4];
   else
@@ -534,7 +476,7 @@ static void set_uint24 (unsigned char *buf, int block, unsigned int value) {
 static char *db_sections[] = {"title", "source", "artist", "genre", "year", "date", "playlist"};
 
 static int build_db_sec_rio (rios_t *rio, unsigned char *buf, int num_tracks, unsigned char *taxi_buf, int section, int start_block) {
-  unsigned char padding[3] = { 0xff, 0xff, 0xff };
+  int padding = 0xffffff;
   int cblock = start_block;
   struct sort_list *tmp_list;
   int i;
@@ -559,7 +501,7 @@ static int build_db_sec_rio (rios_t *rio, unsigned char *buf, int num_tracks, un
   cblock += 2;
 
   strcpy ((char *)&buf[3 * cblock], db_sections[section]);
-  cblock += (strlen(db_sections[section]) + 3 - strlen(db_sections[section]) % 3)/3;
+  cblock += (strlen(db_sections[section]) + 2)/3;
 
   /* these sections are not yet implemented. (year, date, playlist) */
   if (section > 3)
@@ -595,8 +537,7 @@ static int build_db_sec_rio (rios_t *rio, unsigned char *buf, int num_tracks, un
 
       next_letter_block = xblock = cblock;
       
-      memcpy (&buf[3 * cblock++], padding, 3);
-      
+      set_uint24 (buf, cblock++, padding);
       set_uint24 (buf, cblock++, (prev_letter_block) ? prev_letter_block : start_block);
       
       prev_letter_block = xblock;
@@ -611,12 +552,9 @@ static int build_db_sec_rio (rios_t *rio, unsigned char *buf, int num_tracks, un
       set_uint24 (buf, next_block, cblock);
     
     next_block = cblock;
-    memcpy (&buf[3 * cblock++], padding, 3);
-    
-    if (prev_block)
-      set_uint24 (buf, cblock++, prev_block);
-    else
-      memcpy (&buf[3 * cblock++], padding, 3);
+
+    set_uint24 (buf, cblock++, padding);
+    set_uint24 (buf, cblock++, (prev_block) ? prev_block : padding);
 
     count_block = cblock++;
 
@@ -655,7 +593,8 @@ static int build_db_sec_rio (rios_t *rio, unsigned char *buf, int num_tracks, un
 }
 
 static int build_database_rio (rios_t *rio, unsigned char *buf, size_t buf_size) {
-  int i, j;
+  int i;
+  uint j;
 
   unsigned char *taxi_buf;
   
@@ -665,10 +604,13 @@ static int build_database_rio (rios_t *rio, unsigned char *buf, size_t buf_size)
 
   flist_rio_t *flist;
 
-  char db_magic[] = { 0x55, 0x9a, 0x81, 0x03, 0x00, 0x00 };
-  int prev_num = 0;
+  unsigned char db_magic[] = { 0x55, 0x9a, 0x81, 0x03, 0x00, 0x00 };
+  uint prev_num = 0;
   int num_tracks_block;
 
+  /* buf_size is not used (though it should be checked). remove compiler warning */
+  (void) buf_size;
+
   memset (buf, 0xff, 0x3c);
 
   memcpy (buf, db_magic, 6);
@@ -750,12 +692,13 @@ int update_db_rio (rios_t *rio) {
   if (return_type_rio (rio) != RIONITRUS)
     return URIO_SUCCESS;
 
-  rio_log (rio, 0, "update_db_rio: entering...\n");
+  debug("librioutil/song_management.c update_db_rio: entering...");
 
+  /* 8 * RIO_FTS is an arbitrary size for the buffer picked because it should provide enough storage
+     for building the song database. */
   buf = calloc (1, 8 * RIO_FTS);
-
   if (buf == NULL) {
-    rio_log (rio, -errno, "update_db_rio: could not allocate a buffer in which to build the new database.\n");
+    error("librioutil/song_management.c update_db_rio: could not allocate a buffer in which to build the new database.");
 
     return -errno;
   }
@@ -765,7 +708,7 @@ int update_db_rio (rios_t *rio) {
 
   wake_rio (rio);
   if ((ret = send_command_rio (rio, RIO_NINFO, 0, 0)) != URIO_SUCCESS) {
-    rio_log (rio, ret, "update_db_rio: rio did not respond to command.\n");
+    error("librioutil/song_management.c update_db_rio: rio did not respond to command.");
 
     free (buf);
 
@@ -776,7 +719,7 @@ int update_db_rio (rios_t *rio) {
   ret = read_block_rio (rio, NULL, 64, 64);
 
   if (ret != URIO_SUCCESS || strncmp ((char *)rio->buffer, "SRIORDY.", 8) != 0) {
-    rio_log (rio, ret, "update_db_rio: rio is not ready to receive the updated database.\n");
+    error("librioutil/song_management.c update_db_rio: device is not ready to receive the nitrus database: %d", ret);
 
     free (buf);
 
@@ -787,7 +730,7 @@ int update_db_rio (rios_t *rio) {
   ret = read_block_rio (rio, NULL, 64, 64);
 
   if (ret != URIO_SUCCESS || strncmp ((char *)rio->buffer, "SRIODATA", 8) != 0) {
-    rio_log (rio, ret, "update_db_rio: rio is not ready to receive the updated database.\n");
+    error("librioutil/song_management.c update_db_rio: device did not respond as expected: %d", ret);
 
     free (buf);
 
@@ -805,7 +748,7 @@ int update_db_rio (rios_t *rio) {
   ret = read_block_rio (rio, NULL, 64, 64);
 
   if (ret != URIO_SUCCESS || strncmp ((char *)rio->buffer, "SRIODONE", 8) != 0) {
-    rio_log (rio, ret, "update_db_rio: an unknown error occured while trying to write the updated database.\n");
+    error("librioutil/song_management.c update_db_rio: an unknown error occured while trying to write the nitrus database: %d", ret);
 
     free (buf);
 
@@ -814,7 +757,7 @@ int update_db_rio (rios_t *rio) {
 
   free (buf);
 
-  rio_log (rio, 0, "update_db_rio: complete.\n");
+  debug("librioutil/song_management.c update_db_rio: complete.");
 
   return URIO_SUCCESS;
 }
@@ -828,8 +771,10 @@ int update_db_rio (rios_t *rio) {
 static int complete_upload_rio (rios_t *rio, u_int8_t memory_unit, info_page_t info) {
   int ret;
 
-  rio_log (rio, 0, "complete_upload_rio: entering...\n");
+  debug("complete_upload_rio: entering...");
 
+  /* memory_unit is unused. remove compiler warning */
+  (void) memory_unit;
 
   /* Kelly: 08-23-03
      TODO: Set some the data in the RIOT's portion of the
@@ -857,25 +802,52 @@ static int complete_upload_rio (rios_t *rio, u_int8_t memory_unit, info_page_t i
     strncpy((char *)info.data->album2, info.data->album, 48);
   }
 
-  file_to_me (info.data);
+  file_to_arch (info.data);
 
   /* upload the info page */
-  rio_log (rio, 0, "complete_upload_rio: writing file header\n");
+  debug("complete_upload_rio: writing file header");
   
   write_block_rio (rio, (unsigned char *)info.data, sizeof (rio_file_t), "CRIOINFO");
 
-  file_to_me (info.data);
+  file_to_arch (info.data);
   
   
   /* 0x60 command is send by both iTunes and windows softare */
   if ((ret = send_command_rio(rio, 0x60, 0, 0)) != URIO_SUCCESS)
     return ret;
   
-  rio_log (rio, 0, "complete_upload_rio: complete.\n");
+  debug("complete_upload_rio: complete.");
 
   return URIO_SUCCESS;
 }
 
+static int execute_delete_rio (rios_t *rio, u_int8_t memory_unit, rio_file_t *filep) {
+  int ret;
+
+  (void)wake_rio (rio);
+
+  if ((ret = send_command_rio(rio, RIO_DELET, memory_unit, 0)) != URIO_SUCCESS)
+    return ret;
+
+  read_block_rio(rio, NULL, 64, RIO_FTS);
+    
+  if (strncmp((char *)rio->buffer, "SRIODELS", 8) != 0)
+    return -EIO;
+
+  /* correct the endianness of data */
+  file_to_arch(filep);
+
+  if ((ret = write_block_rio(rio, (unsigned char *)filep, RIO_MTS, NULL)) != URIO_SUCCESS)
+    return ret;
+
+  file_to_arch(filep);
+
+  if (strncmp((char *)rio->buffer, "SRIODELD", 8) != 0)
+    return ret;
+
+  return 0;
+}
+
 /*
   delete_file_rio:
 
@@ -891,141 +863,81 @@ static int complete_upload_rio (rios_t *rio, u_int8_t memory_unit, info_page_t i
       - URIO_SUCCESS if the file was delete successfully.
       - < 0 if some error occured.
 */
-int delete_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno) {
-  flist_rio_t *tmp;
+int delete_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t file_num) {
   rio_file_t file;
-  int ret;
+  int ret, file_id;
 
   if ((ret = try_lock_rio (rio)) != 0)
     return ret;
 
-  rio_log (rio, 0, "delete_file_rio: entering...\n");
+  debug("delete_file_rio: entering...");
 
-  for (tmp = rio->info.memory[memory_unit].files ; tmp ; tmp = tmp->next) {
-    if (tmp->num == fileno)
-      break;
-  }
-  
-  if (tmp == NULL)
-    return -1;
-  
-  flist_remove_rio (rio, memory_unit, fileno);
-    
-  update_db_rio (rio);
+  file_id = flist_get_file_id_rio (rio, memory_unit, file_num);
+  if (file_id < 0) {
+    error("librioutil/delete_file_rio: file not found.");
 
-  if ((ret = wake_rio(rio)) != URIO_SUCCESS)
-    UNLOCK(ret);
-
-  if (return_type_rio (rio) != RIONITRUS) {
-    if (get_file_info_rio(rio, &file, memory_unit, tmp->inum) != URIO_SUCCESS)
-      UNLOCK(-1);
-  } else {
-    memset (&file, 0, sizeof (rio_file_t));
-    file.file_no = tmp->rio_num;
+    UNLOCK (file_id);
   }
 
-  if ((ret = send_command_rio(rio, RIO_DELET, memory_unit, 0)) != URIO_SUCCESS)
+  ret = get_file_info_rio(rio, &file, memory_unit, file_id);
+  if (ret != URIO_SUCCESS) {
+    error("librioutil/delete_file_rio: could not get file info");
+
     UNLOCK(ret);
-  
-  if ((int)rio->cmd_buffer[0] != 0) {
-    if ((ret = read_block_rio(rio, NULL, 64, RIO_FTS)) != URIO_SUCCESS)
-      UNLOCK(ret);
-  } else
-    UNLOCK(-EIO);
-    
-  if (strncmp((char *)rio->buffer, "SRIODELS", 8) != 0)
-    UNLOCK(-EIO);
+  }
 
-  /* correct the endianness of data */
-  file_to_me(&file);
+  ret = execute_delete_rio (rio, memory_unit, &file);
+  if (ret != 0) {
+    error("librioutil/delete_file_rio: file deletion failed.");
 
-  if ((ret = write_block_rio(rio, (unsigned char *)&file, RIO_MTS, NULL)) != URIO_SUCCESS)
     UNLOCK(ret);
+  }
 
-  if (strncmp((char *)rio->buffer, "SRIODELD", 8) != 0)
-    UNLOCK(-EIO);
+  /* file deletion successful */
 
+  flist_remove_rio (rio, memory_unit, file_num);
   update_free_intrn_rio (rio, memory_unit);
     
-  rio_log (rio, 0, "delete_file_rio: complete.\n");
+  /* update nitrus database */
+  update_db_rio (rio);
+    
+  debug("delete_file_rio: complete.");
 
   UNLOCK(URIO_SUCCESS);
 }
 
-static int delete_dummy_hdr (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno) {
-  rio_file_t file;
-  int error;
-
-  rio_log (rio, 0, "Clearing dummy header...\n");
-
-  if ((error = get_file_info_rio(rio, &file, memory_unit, fileno)) != URIO_SUCCESS)
-    return error;
-
-  file.size = 0;
-  file.start = 0;
-  file.time = 0;
-  file.bits = 0;
-
-  if ((error = send_command_rio (rio, RIO_DELET, memory_unit, 0)) != URIO_SUCCESS)
-    return error;
-
-  if (strncmp((char *)rio->buffer, "SRIODELS", 8) != 0)
-    return -EIO;
-
-  /* check if the rio responded ok (with a 1) */
-  if ((error = read_block_rio (rio, NULL, 64, RIO_FTS)) != URIO_SUCCESS)
-    return error;
-
-  /* correct the endianness of data */
-  file_to_me(&file);
-
-  if ((error = write_block_rio(rio, (unsigned char *)&file, RIO_MTS, NULL)) != URIO_SUCCESS)
-    return error;
-
-  if (strncmp((char *)rio->buffer, "SRIODELD", 8) != 0)
-    return -EIO;
-
-  return URIO_SUCCESS;
-}
-
-static int upload_dummy_hdr (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno) {
-  rio_file_t file;
+static int upload_dummy_hdr (rios_t *rio, u_int8_t memory_unit, rio_file_t *filep) {
   info_page_t info;
   int error;
   int file_num = flist_first_free_rio (rio, memory_unit);
 
-  rio_log (rio, 0, "upload_dummy_hdr: entering...\n");
+  debug("upload_dummy_hdr: entering...");
 
-  if ((error = get_file_info_rio(rio, &file, memory_unit, fileno)) != URIO_SUCCESS) {
-    rio_log (rio, error, "Error getting file info.\n");
-    return -1;
-  }
+  /* uploading a duplicate file header with this value in the file bits makes
+     the data downloadable on older diamond rios (600, 800, 900) */
+  filep->bits = 0x10000591;
 
-  if (file.bits & 0x00000080)
-    return fileno;
-
-  file.bits = 0x10000591;
-  file.type = 0x00000000;
-  file.file_no = 0;
-  file.bit_rate = 0;
-  file.sample_rate = 0;
+  /* these fields need to be clear to prevent the rio from removing the
+     downloadable bit */
+  filep->type        = 0;
+  filep->file_no     = 0;
+  filep->bit_rate    = 0;
+  filep->sample_rate = 0;
 
   if ((error = init_new_upload_rio(rio, memory_unit)) != URIO_SUCCESS) {
-    rio_log (rio, error, "upload_dummy_hdr(init_upload_rio) error\n");
     abort_transfer_rio(rio);
     return error;
   }
 
-  info.data = &file;
+  info.data = filep;
   info.skip = 0;
 
   if ((error = complete_upload_rio(rio, memory_unit, info))!= URIO_SUCCESS) {
-    rio_log (rio, error, "upload_dummy_hdr(complete_upload_rio) error\n");
     abort_transfer_rio(rio);
     return error;
   }
 
-  rio_log (rio, 0, "upload_dummy_hdr: complete.\n");
+  debug("upload_dummy_hdr: complete.");
 
   return file_num;
 }
@@ -1051,57 +963,58 @@ static int upload_dummy_hdr (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno
   
   It seems that, probably due to the riaa, diamond has
   made it so that wma and mp3 files CAN NOT
-  be downloaded. mp3s that are downloaded will be deleted
-  :(! Keep that in mind (rio600/800/900 only).
+  be downloaded. mp3s that are downloaded will be deleted.
+  (rio600/800/900 only).
 
   All of the newer players from Rio support the download of any
   file on the player!
 */
-int download_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno, char *fileName) {
-  file_list *tmp;
-  rio_file_t file;
-  
-  int ret;
-
+int download_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t file_num, char *file_name) {
+  int i, blocks, size, ret, file_id, player_generation, read_size;
   int downfd;
-  int i, blocks, size;
   
   int download_complete, cr_dummy = -1;
   int mode = S_IRUSR | S_IWUSR | S_IROTH | S_IRGRP;
   int block_size;
 
-  int type, player_generation;
-
   unsigned char dload_buffer[RIO_FTS];
 
-  char *tmp_np;
+  char tmp_np[PATH_MAX];
+  rio_file_t file;
 
   if ((ret = try_lock_rio (rio)) != 0)
     return ret;
 
-  type = return_type_rio (rio);
+  debug("librioutil/song_management.c download_file_rio: entering...");
+
   player_generation = return_generation_rio (rio);
-
-  rio_log (rio, 0, "librioutil/song_management.c download_file_rio: entering...\n");
-
-  /* fetch the file's info */
-  for (tmp = rio->info.memory[memory_unit].files ; tmp ; tmp = tmp->next)
-    if (tmp->num == fileno)
-      break;
   
-  if (!tmp) {
-    rio_log (rio, -ENOENT, "librioutil/song_management.c download_file_rio: no such file.\n");
+  /* get file header data */
+  file_id = flist_get_file_id_rio (rio, memory_unit, file_num);
+  if (file_id < 0) {
+    error("librioutil/song_management.c download_file_rio: file not found: %d", file_id);
 
-    UNLOCK(-ENOENT);
+    UNLOCK(file_id);
   }
 
-  if ((ret = get_file_info_rio(rio, &file, memory_unit, fileno)) != URIO_SUCCESS) {
-    rio_log (rio, ret, "librioutil/song_management.c download_file_rio: error getting file info.\n");
+  if ((ret = get_file_info_rio(rio, &file, memory_unit, file_id)) != URIO_SUCCESS) {
+    error("librioutil/song_management.c download_file_rio: error getting file info: %d", ret);
 
-    return ret;
+    UNLOCK(ret);
   }
 
-  if (return_generation_rio (rio) < 5 && return_version_rio(rio) < 2.0 && type != RIORIOT) {
+  /* generate a filename if one was not supplied */
+  if (file_name == NULL) {
+    (void)flist_get_file_name_rio (rio, memory_unit, file_num, tmp_np, PATH_MAX);
+
+    for ( i = strlen (tmp_np) - 1 ; i > 0 && tmp_np[i] != '\\' && tmp_np[i] != '/' ; i--);
+
+    file_name = &tmp_np[i];
+  }
+  
+
+
+  if (player_generation < 5 && return_version_rio(rio) < 2.0 && return_type_rio (rio) != RIORIOT) {
     /*
       This code is only relevant to older players
       
@@ -1115,70 +1028,65 @@ int download_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno, char
       /* Older players will only allow non-music files to be downloaded. A fake
 	 file header is used to download these files. Such a download will cause
 	 the deletion of the file off of the device. */
-      fileno = upload_dummy_hdr (rio, memory_unit, fileno);
-    }
+      file_id = upload_dummy_hdr (rio, memory_unit, &file);
+      if (file_id < 0) {
+	error("librioutil/song_management.c download_file_rio: error uploading dummy file header.");
+	UNLOCK(file_id);
+      }
   
-    if ((ret = get_file_info_rio(rio, &file, memory_unit, fileno)) != URIO_SUCCESS) {
-      rio_log (rio, ret, "librioutil/song_management.c download_file_rio: could not fetch song info.\n");
-      UNLOCK(ret);
+      if ((ret = get_file_info_rio(rio, &file, memory_unit, file_id)) != URIO_SUCCESS) {
+        error("librioutil/song_management.c download_file_rio: could not fetch song info: %d", ret);
+	UNLOCK(ret);
+      }
     }
   }
 
-  size = tmp->size;
 
-  if ((ret = wake_rio (rio)) != URIO_SUCCESS)
-    UNLOCK(ret);
-  
   /* send the send file command */
+  (void)wake_rio (rio);
+  
   if ((ret = send_command_rio(rio, RIO_READF, memory_unit, 0)) != URIO_SUCCESS)
     UNLOCK(ret);
   
   if ((ret = read_block_rio(rio, NULL, 64, RIO_FTS)) != URIO_SUCCESS)
     UNLOCK(ret);
     
-  /* send the file's info page */
-  file_to_me(&file);
+  /* send file header data */
+  file_to_arch (&file);
   write_block_rio(rio, (unsigned char *)&file, sizeof(rio_file_t), NULL);
+  file_to_arch (&file);
   
   if (memcmp(rio->buffer, "SRIONOFL", 8) == 0) {
     /* file does not exist */
-    rio_log (rio, -ENOENT, "librioutil/song_management.c download_file_rio: (device) no such file\n");
+    error("librioutil/song_management.c download_file_rio: (device) no such file");
+
     UNLOCK(-1);
   }
 
-  if (fileName == NULL) {
-    /* Use the filename stored on the device as the local file name */
-    if (strchr ((char *)tmp->name, ':')) {
-      /* Some files have a full DOS path in their name field. Get the basename
-	 and use that as the file name of the local file. */
-      for ( tmp_np = (char *)&(tmp->name[strlen((char *)tmp->name) - 1]) ; 
-	    tmp_np != tmp->name && *tmp_np != '\\' ; tmp_np--);
-    } else
-      tmp_np = (char *)tmp->name;
-  } else 
-    /* Create a file with a user-specified name. */
-    tmp_np = fileName;
-  
-  rio_log (rio, 0, "librioutil/song_management.c download_file_rio: downloading to file %s\n", tmp_np);
 
-  downfd = creat (tmp_np, mode);
+  /* create local file */
+  debug("librioutil/song_management.c download_file_rio: downloading to file %s", file_name);
+
+  downfd = creat (file_name, mode);
   if (downfd < 0) {
-    rio_log (rio, -errno, "librioutil/song_management.c download_file_rio: could not create local file %s: %s\n",
-	     tmp_np, strerror (errno));
+    error("librioutil/song_management.c download_file_rio: could not create file %s: %s", file_name, strerror (errno));
 
     abort_transfer_rio (rio);
 
     UNLOCK(-1);
   }
 
+
+  size = file.size;
   /* older rios (rio600, rio800, etc) send file data in smaller (4096 byte) chunks. */
   block_size = (player_generation >= 4) ? RIO_FTS : 4096;
   blocks = size/block_size + ((size % block_size) ? 1 : 0);
 
+  if (rio->progress)
+    rio->progress(0, 1, rio->progress_ptr);
+
   /* retrieve file data from the device */
   for (i = 0, download_complete = 0 ; i < blocks ; i++) {
-    int read_size;
-    
     memset (dload_buffer, 0, block_size);
 
     if (rio->abort) {
@@ -1222,7 +1130,7 @@ int download_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno, char
   if (!download_complete) {
     write_cksum_rio (rio, dload_buffer, block_size, "CRIODATA");
   
-    if (return_generation_rio(rio) < 4)
+    if (player_generation < 4)
       read_block_rio(rio, NULL, 64, RIO_FTS);
 
     if (rio->progress)
@@ -1230,19 +1138,17 @@ int download_file_rio (rios_t *rio, u_int8_t memory_unit, u_int32_t fileno, char
   }
 
   close(downfd);
-
-  send_command_rio(rio, 0x65, 0, 0);
-  send_command_rio(rio, 0x66, 0, 0);
   
   if (cr_dummy != -1) {
     /* If a dummy header was uploaded, delete it. An unfortunate side-effect of this
        deletion is the file's data will no longer be available as the device appears
        to free the memory. */
-    delete_dummy_hdr (rio, memory_unit, fileno);
-    /* avoid a possibly bad device state by deleting the original header */
+    execute_delete_rio (rio, memory_unit, &file);
+
+    /* avoid a bad device state by deleting the original file */
     delete_file_rio (rio, memory_unit, cr_dummy);
   }
   
-  rio_log (rio, 0, "librioutil/song_management.c download_file_rio: complete.\n");
+  debug("librioutil/song_management.c download_file_rio: complete.");
   UNLOCK(URIO_SUCCESS);
 }
